"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const perf_hooks_1 = require("perf_hooks");
const TaskLog_1 = require("../TaskLog");
describe("TaskLog", () => {
    test("Example", async () => {
        let nextId = 0;
        const taskLog = new TaskLog_1.TaskLog({
            createUniqueId: () => `${nextId++}`,
            performanceNow: () => perf_hooks_1.performance.now(),
        });
        const history = [];
        taskLog.addListener((event) => history.push(event));
        const value = await taskLog.task({ parentTaskId: null, label: "task 1", kind: "add" }, async (parentTaskId) => {
            await new Promise((resolve) => setTimeout(resolve, 100));
            return ((await taskLog.task({ parentTaskId, label: "childAdd", kind: "add" }, async () => 1 + 4)) +
                (await taskLog.task({ parentTaskId, label: "childMult", kind: "mult" }, async () => 5 * 3)));
        });
        taskLog.removeAllListeners();
        expect(value).toEqual(20);
        expect(history).toEqual([
            {
                taskId: "0",
                kind: "begin",
                timestamp: expect.any(Number),
                performanceNow: expect.any(Number),
                meta: { parentTaskId: null, label: "task 1", kind: "add" },
            },
            {
                taskId: "1",
                kind: "begin",
                timestamp: expect.any(Number),
                performanceNow: expect.any(Number),
                meta: { parentTaskId: "0", label: "childAdd", kind: "add" },
            },
            {
                taskId: "1",
                meta: { parentTaskId: "0", label: "childAdd", kind: "add" },
                timestamp: expect.any(Number),
                performanceNow: expect.any(Number),
                duration: expect.any(Number),
                kind: "end",
                result: { kind: "value", value: 5 },
            },
            {
                taskId: "2",
                kind: "begin",
                timestamp: expect.any(Number),
                performanceNow: expect.any(Number),
                meta: { parentTaskId: "0", label: "childMult", kind: "mult" },
            },
            {
                taskId: "2",
                meta: { parentTaskId: "0", label: "childMult", kind: "mult" },
                timestamp: expect.any(Number),
                performanceNow: expect.any(Number),
                duration: expect.any(Number),
                kind: "end",
                result: { kind: "value", value: 15 },
            },
            {
                taskId: "0",
                meta: { parentTaskId: null, label: "task 1", kind: "add" },
                timestamp: expect.any(Number),
                performanceNow: expect.any(Number),
                duration: expect.any(Number),
                kind: "end",
                result: { kind: "value", value: 20 },
            },
        ]);
    });
    test("Full tests", async () => {
        let nextId = 0;
        const taskLog = new TaskLog_1.TaskLog({
            createUniqueId: () => `${nextId++}`,
            performanceNow: () => perf_hooks_1.performance.now(),
        });
        const sleep = (delay) => new Promise((resolve) => setTimeout(resolve, delay));
        const history = [];
        taskLog.addListener((event) => history.push(event));
        const history1 = [];
        const history2 = [];
        const history3 = [];
        const historyThreshold = [];
        const listener1 = taskLog.addListener((event) => history1.push(event));
        const listener2 = taskLog.addListener((event) => history2.push(event));
        taskLog.addListener((event) => {
            if (event.kind === "end" && event.duration > 50) {
                historyThreshold.push(event);
            }
        });
        const value1 = await taskLog.task({ parentTaskId: null, kind: "task", label: "task 1" }, async (parentTaskId) => {
            const v1 = await taskLog.task({
                parentTaskId,
                kind: "subtask",
                label: "subtask 1.1",
                expect: "value",
            }, async () => await sleep(10).then(() => 1));
            const v2 = await taskLog
                .task({
                parentTaskId: "0",
                kind: "subtask",
                label: "subtask 1.2",
                expect: "error",
            }, async () => {
                throw new Error("subtask 1.2 error");
            })
                .catch(() => null);
            return [v1, v2];
        });
        taskLog.addListener((event) => history3.push(event));
        taskLog.removeListener(listener2);
        const value2 = await taskLog
            .task({ parentTaskId: null, kind: "task", label: "task 2" }, async (parentTaskId) => {
            await taskLog.task({
                parentTaskId,
                kind: "subtask",
                label: "subtask 2.1",
                expect: "value",
            }, async () => 3);
            await taskLog.task({
                parentTaskId,
                kind: "subtask",
                label: "subtask 2.2",
                expect: "value",
            }, async () => await sleep(100).then(() => 4));
            throw new Error("task 2 error");
        })
            .catch(() => null);
        taskLog.removeListener(listener1);
        const value3 = await taskLog.task({ parentTaskId: null, kind: "task", label: "task 3" }, async (parentTaskId) => {
            const [v1, v2] = await taskLog.task({
                parentTaskId,
                kind: "subtask",
                label: "subtask 3.1",
                expect: "value",
            }, async (parentTaskId) => {
                const v1 = await taskLog.task({
                    parentTaskId,
                    kind: "subtask",
                    label: "subtask 3.1.1",
                    expect: "value",
                }, async () => "a");
                const v2 = "b";
                return [v1, v2];
            });
            const v3 = await taskLog.task({
                parentTaskId,
                kind: "subtask",
                label: "subtask 3.2",
                expect: "value",
            }, async () => "c");
            await sleep(100);
            return [v1, v2, v3];
        });
        taskLog.removeAllListeners();
        const value4 = await taskLog.task({ parentTaskId: null, kind: "task", label: "task 4" }, async (parentTaskId) => {
            const v1 = await taskLog.task({
                parentTaskId,
                kind: "subtask",
                label: "subtask 4.1",
                expect: "value",
            }, async () => 5);
            const v2 = await taskLog.task({
                parentTaskId,
                kind: "subtask",
                label: "subtask 4.1",
                expect: "value",
            }, async () => 6);
            return [v1, v2];
        });
        expect(value1).toEqual([1, null]);
        expect(value2).toEqual(null);
        expect(value3).toEqual(["a", "b", "c"]);
        expect(value4).toEqual([5, 6]);
        expect(history1).toHaveLength(12);
        expect(history2).toHaveLength(6);
        expect(history3).toHaveLength(14);
        expect(historyThreshold).toHaveLength(3);
        expect(history1).toEqual([
            {
                taskId: "0",
                kind: "begin",
                timestamp: expect.any(Number),
                performanceNow: expect.any(Number),
                meta: { parentTaskId: null, kind: "task", label: "task 1" },
            },
            {
                taskId: "1",
                kind: "begin",
                timestamp: expect.any(Number),
                performanceNow: expect.any(Number),
                meta: {
                    parentTaskId: "0",
                    kind: "subtask",
                    label: "subtask 1.1",
                    expect: "value",
                },
            },
            {
                taskId: "1",
                meta: {
                    parentTaskId: "0",
                    kind: "subtask",
                    label: "subtask 1.1",
                    expect: "value",
                },
                timestamp: expect.any(Number),
                performanceNow: expect.any(Number),
                duration: expect.any(Number),
                kind: "end",
                result: { kind: "value", value: 1 },
            },
            {
                taskId: "2",
                kind: "begin",
                timestamp: expect.any(Number),
                performanceNow: expect.any(Number),
                meta: {
                    parentTaskId: "0",
                    kind: "subtask",
                    label: "subtask 1.2",
                    expect: "error",
                },
            },
            {
                taskId: "2",
                meta: {
                    parentTaskId: "0",
                    kind: "subtask",
                    label: "subtask 1.2",
                    expect: "error",
                },
                timestamp: expect.any(Number),
                performanceNow: expect.any(Number),
                duration: expect.any(Number),
                kind: "end",
                result: {
                    kind: "error",
                    error: expect.any(Error),
                },
            },
            {
                taskId: "0",
                meta: {
                    parentTaskId: null,
                    kind: "task",
                    label: "task 1",
                },
                timestamp: expect.any(Number),
                performanceNow: expect.any(Number),
                duration: expect.any(Number),
                kind: "end",
                result: { kind: "value", value: [1, null] },
            },
            {
                taskId: "3",
                kind: "begin",
                timestamp: expect.any(Number),
                performanceNow: expect.any(Number),
                meta: {
                    parentTaskId: null,
                    kind: "task",
                    label: "task 2",
                },
            },
            {
                taskId: "4",
                kind: "begin",
                timestamp: expect.any(Number),
                performanceNow: expect.any(Number),
                meta: {
                    parentTaskId: "3",
                    kind: "subtask",
                    label: "subtask 2.1",
                    expect: "value",
                },
            },
            {
                taskId: "4",
                meta: {
                    parentTaskId: "3",
                    kind: "subtask",
                    label: "subtask 2.1",
                    expect: "value",
                },
                timestamp: expect.any(Number),
                performanceNow: expect.any(Number),
                duration: expect.any(Number),
                kind: "end",
                result: { kind: "value", value: 3 },
            },
            {
                taskId: "5",
                kind: "begin",
                timestamp: expect.any(Number),
                performanceNow: expect.any(Number),
                meta: {
                    parentTaskId: "3",
                    kind: "subtask",
                    label: "subtask 2.2",
                    expect: "value",
                },
            },
            {
                taskId: "5",
                meta: {
                    parentTaskId: "3",
                    kind: "subtask",
                    label: "subtask 2.2",
                    expect: "value",
                },
                timestamp: expect.any(Number),
                performanceNow: expect.any(Number),
                duration: expect.any(Number),
                kind: "end",
                result: { kind: "value", value: 4 },
            },
            {
                taskId: "3",
                meta: {
                    parentTaskId: null,
                    kind: "task",
                    label: "task 2",
                },
                timestamp: expect.any(Number),
                performanceNow: expect.any(Number),
                duration: expect.any(Number),
                kind: "end",
                result: {
                    kind: "error",
                    error: expect.any(Error),
                },
            },
        ]);
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        expect(history1[4].result.error.message).toEqual("subtask 1.2 error");
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        expect(history1[11].result.error.message).toEqual("task 2 error");
        expect(history3.slice(6)).toEqual([
            {
                taskId: "6",
                kind: "begin",
                timestamp: expect.any(Number),
                performanceNow: expect.any(Number),
                meta: {
                    parentTaskId: null,
                    kind: "task",
                    label: "task 3",
                },
            },
            {
                taskId: "7",
                kind: "begin",
                timestamp: expect.any(Number),
                performanceNow: expect.any(Number),
                meta: {
                    parentTaskId: "6",
                    kind: "subtask",
                    label: "subtask 3.1",
                    expect: "value",
                },
            },
            {
                taskId: "8",
                kind: "begin",
                timestamp: expect.any(Number),
                performanceNow: expect.any(Number),
                meta: {
                    parentTaskId: "7",
                    kind: "subtask",
                    label: "subtask 3.1.1",
                    expect: "value",
                },
            },
            {
                taskId: "8",
                meta: {
                    parentTaskId: "7",
                    kind: "subtask",
                    label: "subtask 3.1.1",
                    expect: "value",
                },
                timestamp: expect.any(Number),
                performanceNow: expect.any(Number),
                duration: expect.any(Number),
                kind: "end",
                result: { kind: "value", value: "a" },
            },
            {
                taskId: "7",
                meta: {
                    parentTaskId: "6",
                    kind: "subtask",
                    label: "subtask 3.1",
                    expect: "value",
                },
                timestamp: expect.any(Number),
                performanceNow: expect.any(Number),
                duration: expect.any(Number),
                kind: "end",
                result: { kind: "value", value: ["a", "b"] },
            },
            {
                taskId: "9",
                kind: "begin",
                timestamp: expect.any(Number),
                performanceNow: expect.any(Number),
                meta: {
                    parentTaskId: "6",
                    kind: "subtask",
                    label: "subtask 3.2",
                    expect: "value",
                },
            },
            {
                taskId: "9",
                meta: {
                    parentTaskId: "6",
                    kind: "subtask",
                    label: "subtask 3.2",
                    expect: "value",
                },
                timestamp: expect.any(Number),
                performanceNow: expect.any(Number),
                duration: expect.any(Number),
                kind: "end",
                result: { kind: "value", value: "c" },
            },
            {
                taskId: "6",
                meta: {
                    parentTaskId: null,
                    kind: "task",
                    label: "task 3",
                },
                timestamp: expect.any(Number),
                performanceNow: expect.any(Number),
                duration: expect.any(Number),
                kind: "end",
                result: { kind: "value", value: ["a", "b", "c"] },
            },
        ]);
    });
});
//# sourceMappingURL=TaskLog.test.js.map