interface ITaskEventHeader<TaskMeta> {
  readonly taskId: string;
  readonly meta: TaskMeta;
  readonly timestamp: number;
  readonly performanceNow: number;
}

interface ITaskEventBegin<TaskMeta> extends ITaskEventHeader<TaskMeta> {
  readonly kind: "begin";
}

interface ITaskEventEnd<TaskMeta> extends ITaskEventHeader<TaskMeta> {
  readonly kind: "end";
  readonly duration: number;
  readonly result:
    | {
        readonly kind: "value";
        readonly value: unknown;
      }
    | {
        readonly kind: "error";
        readonly error: Error;
      };
}

export type TaskEvent<TaskMeta> =
  | ITaskEventBegin<TaskMeta>
  | ITaskEventEnd<TaskMeta>;

type TaskListener<TaskMeta> = (event: TaskEvent<TaskMeta>) => void;

interface ITaskLogProps {
  readonly performanceNow: () => number;
  readonly createUniqueId: () => string;
}

export class TaskLog<TaskMeta> {
  private readonly props: ITaskLogProps;
  private readonly listeners: Set<TaskListener<TaskMeta>>;

  public constructor(props: ITaskLogProps) {
    this.props = props;
    this.listeners = new Set();
  }

  private readonly emit = (event: TaskEvent<TaskMeta>): void => {
    for (const listener of this.listeners) {
      listener(event);
    }
  };

  public readonly addListener = (
    listener: TaskListener<TaskMeta>,
  ): TaskListener<TaskMeta> => {
    this.listeners.add(listener);
    return listener;
  };

  public readonly removeListener = (listener: TaskListener<TaskMeta>): void => {
    this.listeners.delete(listener);
  };

  public readonly removeAllListeners = (): void => {
    this.listeners.clear();
  };

  public readonly task = async <T>(
    meta: TaskMeta,
    fn: (taskId: string) => Promise<T>,
  ): Promise<T> => {
    const taskId = this.props.createUniqueId();
    const beginTimestamp = Date.now();
    const beginPerformanceNow = this.props.performanceNow();
    this.emit({
      taskId,
      kind: "begin",
      timestamp: beginTimestamp,
      performanceNow: beginPerformanceNow,
      meta,
    });
    try {
      const value = await fn(taskId);
      const endTimestamp = Date.now();
      const endPerformanceNow = this.props.performanceNow();
      this.emit({
        taskId,
        meta,
        timestamp: endTimestamp,
        performanceNow: endPerformanceNow,
        duration: endPerformanceNow - beginPerformanceNow,
        kind: "end",
        result: {
          kind: "value",
          value,
        },
      });
      return value;
    } catch (error) {
      const endTimestamp = Date.now();
      const endPerformanceNow = this.props.performanceNow();
      this.emit({
        taskId,
        meta,
        timestamp: endTimestamp,
        performanceNow: endPerformanceNow,
        duration: endPerformanceNow - beginPerformanceNow,
        kind: "end",
        result: {
          kind: "error",
          error,
        },
      });
      throw error;
    }
  };
}
